"use strict";

import isUndefined from 'lodash/isUndefined';
import React, { useRef, useCallback, useEffect, memo } from 'react';
import { Animated, View } from 'react-native';
import Card from "../card/index.js";
import Cell from "../cell/index.js";
import Divider from "../divider/index.js";
import { easing, getDefaultValue, getArrowOutline } from "../helpers/index.js";
import { usePersistFn, useControllableValue } from "../hooks/index.js";
import Theme from "../theme/index.js";
import { varCreator, styleCreator } from "./style.js";

/**
 * Collapse 折叠面板
 */
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
const Collapse = ({
  children,
  theme,
  title,
  titleStyle,
  titleTextStyle,
  iconStyle,
  iconColor,
  iconSize,
  bodyStyle,
  renderTitle,
  renderTitleExtra,
  renderBody,
  type = 'cell',
  onAnimationEnd,
  bodyPadding = true,
  headerDivider = true,
  bodyDivider,
  lazyRender = true,
  square = true,
  testID,
  ...restProps
}) => {
  const [CV, STYLES] = Theme.useStyle({
    varCreator,
    styleCreator,
    theme
  });
  bodyDivider = getDefaultValue(bodyDivider, type === 'cell');
  const [collapse, onCollapse] = useControllableValue(restProps, {
    defaultValuePropName: 'defaultCollapse',
    valuePropName: 'collapse',
    trigger: 'onCollapse',
    defaultValue: false
  });
  const onAnimationEndPersistFn = usePersistFn(v => {
    onAnimationEnd?.(v);
  });
  /** 记录当前是否可见，在不断 onLayout 的时候可以有一个判断的依据 */
  const Visible = useRef(collapse);
  const BodyHeight = useRef(0);
  const MountedRef = useRef(false);
  const AnimatedValue = useRef(new Animated.Value(0)).current;
  const toggleBody = useCallback((v, immediately) => {
    const action = Animated.timing(AnimatedValue, {
      toValue: v ? BodyHeight.current : 0,
      duration: immediately ? 0 : CV.collapse_transition_duration,
      useNativeDriver: false,
      easing: v ? easing.easeOutCirc : easing.easeInCubic
    });
    action.start(({
      finished
    }) => {
      if (finished) {
        onAnimationEndPersistFn(v);
      }
    });
  }, [AnimatedValue, onAnimationEndPersistFn, CV.collapse_transition_duration]);

  // 初始化好组件
  useEffect(() => {
    MountedRef.current = true;
  }, []);
  useEffect(() => {
    // 同步当前的状态
    Visible.current = collapse;
    toggleBody(collapse, false);
  }, [collapse, toggleBody]);
  const onPressTitle = useCallback(() => {
    onCollapse(!Visible.current);
  }, [onCollapse]);
  const onLayoutBody = useCallback(e => {
    // 有点疑惑，折叠的过程中，高度在动态变化，通过 absolute 布局解决无法完全渲染
    BodyHeight.current = e.nativeEvent.layout.height;
    // 展开的过程中才会存在动态高度
    if (Visible.current) {
      // 当收齐的时候已知高度
      toggleBody(Visible.current, Visible.current);
    }
  }, [toggleBody]);
  const ArrowOutline = getArrowOutline(collapse ? 'up' : 'down');
  const arrowJSX = /*#__PURE__*/_jsx(ArrowOutline, {
    style: iconStyle,
    color: !isUndefined(iconColor) ? iconColor : CV.collapse_icon_color,
    size: !isUndefined(iconSize) ? iconSize : CV.collapse_icon_size
  });
  const titleJSX = renderTitle ? renderTitle(collapse) : title;
  const titleExtraJSX = renderTitleExtra ? renderTitleExtra(collapse, arrowJSX) : arrowJSX;
  const bodyJSX = lazyRender && !MountedRef.current && !collapse ? null : renderBody ? renderBody() : children;
  const ctxJSX = /*#__PURE__*/_jsx(Animated.View, {
    style: [STYLES.collapse, {
      height: AnimatedValue
    }],
    children: /*#__PURE__*/_jsxs(View, {
      collapsable: false,
      style: STYLES.body,
      onLayout: onLayoutBody,
      children: [/*#__PURE__*/_jsx(View, {
        style: [bodyPadding ? STYLES.body_padding : undefined, bodyStyle],
        children: bodyJSX
      }), bodyDivider ? /*#__PURE__*/_jsx(Divider, {
        type: "light",
        style: STYLES.divider
      }) : null]
    })
  });
  if (type === 'card') {
    return /*#__PURE__*/_jsx(Card, {
      square: square,
      title: titleJSX,
      extra: titleExtraJSX,
      headerDivider: headerDivider,
      titleStyle: titleStyle,
      titleTextStyle: titleTextStyle,
      bodyPadding: false,
      onPressHeader: onPressTitle,
      testID: testID,
      children: ctxJSX
    });
  }
  return /*#__PURE__*/_jsxs(_Fragment, {
    children: [/*#__PURE__*/_jsx(Cell, {
      title: titleJSX,
      style: titleStyle,
      titleTextStyle: titleTextStyle,
      valueExtra: titleExtraJSX,
      onPress: onPressTitle,
      divider: headerDivider,
      testID: testID
    }), ctxJSX]
  });
};
export default /*#__PURE__*/memo(Collapse);
//# sourceMappingURL=collapse.js.map