"use strict";

import { CrossOutline } from '@fruits-chain/icons-react-native';
import isNil from 'lodash/isNil';
import React, { useRef, useCallback, memo, isValidElement } from 'react';
import { View, Text, Animated } from 'react-native';
import Button from "../button/index.js";
import { getDefaultValue, easing, renderTextLikeJSX } from "../helpers/index.js";
import { usePersistFn } from "../hooks/index.js";
import Locale from "../locale/index.js";
import Popup from "../popup/popup.js";
import Theme from "../theme/index.js";
import { varCreator, styleCreator } from "./style.js";
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const defaultOnRequestClose = () => {
  return true;
};

/**
 * Dialog 弹出框
 * @description 弹出模态框，常用于消息提示、消息确认，或在当前页面内完成特定的交互操作。
 * @description 弹出框组件支持函数调用和组件调用两种方式。
 */
const Dialog = ({
  children,
  style,
  theme,
  title,
  message,
  width,
  messageAlign = 'center',
  showConfirmButton = true,
  showCancelButton = false,
  confirmButtonText,
  cancelButtonText,
  confirmButtonColor,
  confirmButtonTextBold = true,
  cancelButtonColor,
  cancelButtonTextBold = false,
  confirmButtonLoading = false,
  cancelButtonLoading = false,
  showClose = false,
  onPressClose,
  buttonReverse = false,
  onPressCancel,
  onPressConfirm,
  duration,
  onOpen: onOpenFn,
  onClose: onCloseFn,
  onRequestClose = defaultOnRequestClose,
  ...resetProps
}) => {
  const fadeAnim = useRef(new Animated.Value(0)).current;
  const fadeInstance = useRef(null);
  const locale = Locale.useLocale().Dialog;
  const [CV, STYLES] = Theme.useStyle({
    varCreator,
    styleCreator,
    theme
  });
  width = getDefaultValue(width, CV.dialog_width);
  duration = getDefaultValue(duration, CV.dialog_transition);
  const showDialog = useCallback(show => {
    if (fadeInstance.current) {
      fadeInstance.current.stop();
      fadeInstance.current = null;
    }
    fadeInstance.current = Animated.timing(fadeAnim,
    // 动画中的变量值
    {
      toValue: show ? 1 : 0,
      duration: duration,
      useNativeDriver: true,
      easing: show ? easing.easeOutCirc : easing.easeInCubic
    });
    fadeInstance.current.start();
  }, [duration, fadeAnim]);
  const onOpenPersistFn = usePersistFn(() => {
    showDialog(true);
    onOpenFn?.();
  });
  const onClosePersistFn = usePersistFn(() => {
    showDialog(false);
    onCloseFn?.();
  });
  const titleJSX = renderTextLikeJSX(title, [STYLES.title_text, isNil(message) ? STYLES.title_isolated : null]);
  const messageJSX = !isNil(message) ? /*#__PURE__*/isValidElement(message) ? message : /*#__PURE__*/_jsx(Text, {
    style: [STYLES.message_text, {
      textAlign: messageAlign
    }],
    children: message
  }) : null;
  const cancelButtonProps = {
    color: cancelButtonColor || CV.dialog_cancel_button_text_color,
    text: cancelButtonText ?? locale.cancelButtonText,
    loading: cancelButtonLoading,
    onPress: onPressCancel
  };
  const confirmButtonProps = {
    color: confirmButtonColor || CV.dialog_confirm_button_text_color,
    text: confirmButtonText ?? locale.confirmButtonText,
    loading: confirmButtonLoading,
    onPress: onPressConfirm
  };

  // TODO 优化逆转按钮变量变换
  const _showCancelButton = buttonReverse ? showConfirmButton : showCancelButton;
  const _showConfirmButton = buttonReverse ? showCancelButton : showConfirmButton;
  const _cancelButtonTextBold = buttonReverse ? confirmButtonTextBold : cancelButtonTextBold;
  const _confirmButtonTextBold = buttonReverse ? cancelButtonTextBold : confirmButtonTextBold;
  const _cancelButtonProps = buttonReverse ? confirmButtonProps : cancelButtonProps;
  const _confirmButtonProps = buttonReverse ? cancelButtonProps : confirmButtonProps;
  return /*#__PURE__*/_jsx(Popup, {
    ...resetProps,
    duration: duration,
    onOpen: onOpenPersistFn,
    onClose: onClosePersistFn,
    onRequestClose: onRequestClose,
    children: /*#__PURE__*/_jsxs(Animated.View, {
      style: [STYLES.dialog, style, {
        width,
        transform: [{
          scale: fadeAnim.interpolate({
            inputRange: [0, 0.01, 0.98, 1],
            outputRange: [0, 0.9, 1.02, 1]
          })
        }]
      }],
      children: [showClose ? /*#__PURE__*/_jsx(CrossOutline, {
        style: STYLES.close,
        onPress: onPressClose,
        color: CV.dialog_close_color,
        size: CV.dialog_close_size
      }) : null, titleJSX, titleJSX ? messageJSX : /*#__PURE__*/_jsx(View, {
        style: STYLES.content_isolated,
        children: messageJSX
      }), children, _showCancelButton || _showConfirmButton ? /*#__PURE__*/_jsxs(View, {
        style: STYLES.footer,
        children: [_showCancelButton ? /*#__PURE__*/_jsx(Button, {
          ..._cancelButtonProps,
          type: "link",
          size: "xl",
          square: true,
          style: STYLES.btn,
          textStyle: _cancelButtonTextBold ? STYLES.btn_text_bold : null
        }) : null, _showConfirmButton ? /*#__PURE__*/_jsx(Button, {
          ..._confirmButtonProps,
          type: "link",
          size: "xl",
          square: true,
          style: [STYLES.btn, _showCancelButton ?
          // eslint-disable-next-line react-native/no-inline-styles
          {
            // react-native-web 覆盖原 button 样式
            borderLeftWidth: 1,
            borderColor: CV.dialog_footer_divider_color
          } : null],
          textStyle: _confirmButtonTextBold ? STYLES.btn_text_bold : null
        }) : null]
      }) : null]
    })
  });
};
export default /*#__PURE__*/memo(Dialog);
//# sourceMappingURL=dialog.js.map