"use strict";

import isBoolean from 'lodash/isBoolean';
import isNil from 'lodash/isNil';
import { memo, useMemo, useRef } from 'react';
import { View, Animated, useWindowDimensions, PanResponder } from 'react-native';
import { useSafeAreaInsets } from 'react-native-safe-area-context';
import Divider from "../divider/index.js";
import { getDefaultValue, renderTextLikeJSX } from "../helpers/index.js";
import Theme from "../theme/index.js";
import { varCreator, styleCreator } from "./style.js";
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const FloatingPanel = ({
  theme,
  anchorStart,
  anchorEnd,
  title,
  titleDivider = true,
  offsetThreshold = 0.2,
  zIndex = 10,
  draggingOnContent = true,
  onAnimationEnd,
  _onMoveShouldSetPanResponder,
  children,
  ...restProps
}) => {
  const insets = useSafeAreaInsets();
  const {
    height
  } = useWindowDimensions();
  const [CV, STYLES] = Theme.useStyle({
    varCreator,
    styleCreator,
    theme
  });
  const hasTitle = !isNil(title);
  const _anchorStart = getDefaultValue(anchorStart, insets.bottom + CV.floating_panel_header_padding * 2 + (hasTitle ? CV.floating_panel_header_text_font_size : 0));
  const _anchorEnd = getDefaultValue(anchorEnd, height - insets.top);
  const yMax = _anchorEnd - _anchorStart;
  const yMin = 0;
  const PanAnimated = useRef(new Animated.Value(yMax));
  const Anchor = useRef(_anchorEnd - _anchorStart);
  const panResponder = useMemo(() => {
    return PanResponder.create({
      onMoveShouldSetPanResponder: (_, gestureState) => {
        const stop = _onMoveShouldSetPanResponder?.(_, gestureState);
        if (isBoolean(stop)) {
          return stop;
        }
        if (Anchor.current === yMin && gestureState.dy < 0) {
          return false;
        }
        return true;
      },
      onPanResponderMove: (e, gestureState) => {
        let newValue = Anchor.current + gestureState.dy;
        if (newValue > yMax) {
          newValue = yMax;
          e.stopPropagation();
        }
        if (newValue < yMin) {
          newValue = yMin;
          e.stopPropagation();
        }
        PanAnimated.current.setValue(newValue);
      },
      onPanResponderRelease: (_, gestureState) => {
        const triggered = Math.abs(gestureState.dy) >= _anchorEnd * offsetThreshold;
        const newValue = triggered ? gestureState.dy < 0 ? yMin : yMax : Anchor.current;
        Animated.timing(PanAnimated.current, {
          toValue: newValue,
          // 设置动画的属性值
          useNativeDriver: true,
          duration: 200
        }).start(({
          finished
        }) => {
          if (finished) {
            Anchor.current = newValue;
            onAnimationEnd?.(newValue === yMin);
          }
        });
      }
    });
  }, [_anchorEnd, _onMoveShouldSetPanResponder, offsetThreshold, onAnimationEnd, yMax]);
  return /*#__PURE__*/_jsxs(Animated.View, {
    ...restProps,
    ...(draggingOnContent ? panResponder.panHandlers : undefined),
    style: [STYLES.floatingPanel, {
      zIndex,
      height: _anchorEnd,
      transform: [{
        translateY: PanAnimated.current
      }]
    }, restProps.style],
    children: [/*#__PURE__*/_jsxs(View, {
      style: STYLES.header,
      ...(!draggingOnContent ? panResponder.panHandlers : undefined),
      children: [renderTextLikeJSX(title, STYLES.headerText), /*#__PURE__*/_jsx(View, {
        style: STYLES.indicator
      })]
    }), titleDivider ? /*#__PURE__*/_jsx(Divider, {}) : null, children]
  });
};
export default /*#__PURE__*/memo(FloatingPanel);
//# sourceMappingURL=floating-panel.js.map