"use strict";

import isUndefined from 'lodash/isUndefined';
import React, { useEffect, useRef, useMemo, memo } from 'react';
import { TouchableWithoutFeedback, Animated, View } from 'react-native';
import { getDefaultValue, callInterceptor, renderTextLikeJSX } from "../helpers/index.js";
import { useControllableValue, usePersistFn, useDifferentState } from "../hooks/index.js";
import LoadingCircular from "../loading/loading-circular.js";
import { varCreator as varCreatorLoading } from "../loading/style.js";
import Theme from "../theme/index.js";
import { varCreator, styleCreator } from "./style.js";

/**
 * Switch 开关
 * @description 用于在打开和关闭状态之间进行切换。
 */
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
function Switch({
  theme,
  size,
  disabled = false,
  loading = false,
  activeValue = true,
  inactiveValue = false,
  activeColor,
  inactiveColor,
  activeChildren,
  inactiveChildren,
  onPress,
  beforeChange,
  testID,
  ...restProps
}) {
  const translateX = useRef(new Animated.Value(0));
  const [value, onChange] = useControllableValue(restProps, {
    defaultValue: inactiveValue
  });
  const [CV, STYLES] = Theme.useStyle({
    varCreator,
    styleCreator,
    theme
  });
  const [CV_LOADING] = Theme.useStyle({
    varCreator: varCreatorLoading
  });
  const unitSize = getDefaultValue(size, CV.switch_size);
  const nodeEdgeDistance = 2;
  const [switchWidth, setSwitchWidth] = useDifferentState(unitSize * CV.switch_width_ratio);
  const [switchHeight, nodeSize, translateXValueEnd, translateXValueStart] = useMemo(() => {
    const _switchHeight = unitSize * CV.switch_height_ratio;
    const _nodeSize = unitSize * CV.switch_node_size_ratio;
    const _isInnerNode = _switchHeight - _nodeSize < nodeEdgeDistance * 2;
    const _nodeRealSize = _isInnerNode ? _nodeSize - nodeEdgeDistance * 2 : _nodeSize;
    const _innerPadding = _isInnerNode ? nodeEdgeDistance : (_switchHeight - _nodeSize) / 2;
    const _translateXValueEnd = switchWidth - _nodeRealSize - _innerPadding;
    const _translateXValueStart = _innerPadding;
    return [_switchHeight, _nodeRealSize, _translateXValueEnd, _translateXValueStart];
  }, [CV.switch_height_ratio, CV.switch_node_size_ratio, switchWidth, unitSize]);
  const active = value === activeValue;
  const onPressTouchable = () => {
    onPress?.();
    if (!disabled && !loading) {
      const newValue = active ? inactiveValue : activeValue;
      callInterceptor(beforeChange, {
        args: [newValue],
        done: () => {
          onChange(newValue);
        }
      });
    }
  };
  useEffect(() => {
    const actionValue = Animated.timing(translateX.current,
    // 动画中的变量值
    {
      toValue: active ? 1 : 0,
      duration: CV.switch_transition_duration,
      useNativeDriver: true
    });
    actionValue.start();
    return () => {
      // 停止动画
      if (actionValue) {
        actionValue.stop();
      }
    };
  }, [active, CV.switch_transition_duration]);
  const switchStyles = [STYLES.switch, {
    minWidth: switchWidth,
    height: switchHeight,
    borderRadius: switchHeight / 2,
    // 当前过渡不支持 color/backgroundColor
    // 参考：https://stackoverflow.com/a/60586628
    backgroundColor: active ? activeColor || CV.switch_on_background_color : inactiveColor || CV.switch_background_color
  }, disabled ? STYLES.disabled : {}];
  const nodeStyleSummary = [STYLES.node, {
    top: nodeEdgeDistance,
    width: nodeSize,
    height: nodeSize,
    borderRadius: nodeSize / 2,
    transform: [{
      translateX: translateX.current.interpolate({
        inputRange: [0, 1],
        outputRange: [translateXValueStart, translateXValueEnd]
      })
    }]
  }];
  const childrenMinEdgeDistance = switchHeight / 3;
  const childrenMaxEdgeDistance = nodeSize + nodeEdgeDistance * 3;
  const activeChildrenStyle = {
    height: switchHeight,
    paddingLeft: childrenMinEdgeDistance,
    paddingRight: childrenMaxEdgeDistance,
    justifyContent: 'center',
    alignItems: 'center',
    transform: [{
      translateX: translateX.current.interpolate({
        inputRange: [0, 1],
        outputRange: [-switchWidth, 0]
      })
    }]
  };
  const inactiveChildrenStyle = {
    marginTop: -switchHeight,
    height: switchHeight,
    paddingLeft: childrenMaxEdgeDistance,
    paddingRight: childrenMinEdgeDistance,
    justifyContent: 'center',
    alignItems: 'center',
    transform: [{
      translateX: translateX.current.interpolate({
        inputRange: [0, 1],
        outputRange: [0, switchWidth]
      })
    }]
  };
  const onLayoutChildren = usePersistFn(e => {
    setSwitchWidth(v => Math.max(v, e.nativeEvent.layout.width));
  });
  const activeChildrenJSX = renderTextLikeJSX(activeChildren, [STYLES.children_text]);
  const inactiveChildrenJSX = renderTextLikeJSX(inactiveChildren, [STYLES.children_text]);
  return /*#__PURE__*/_jsx(TouchableWithoutFeedback, {
    onPress: onPressTouchable,
    testID: testID,
    children: /*#__PURE__*/_jsx(View, {
      style: STYLES.switch_wrap,
      collapsable: false,
      children: /*#__PURE__*/_jsxs(View, {
        style: switchStyles,
        collapsable: false,
        children: [/*#__PURE__*/_jsx(Animated.View, {
          style: nodeStyleSummary,
          children: loading ? /*#__PURE__*/_jsx(LoadingCircular, {
            size: nodeSize / 4 * 3,
            color: active ? !isUndefined(activeColor) ? activeColor : CV.switch_on_background_color : !isUndefined(inactiveColor) ? inactiveColor : CV_LOADING.loading_text_color
          }) : null
        }), /*#__PURE__*/_jsxs(View, {
          style: STYLES.children_wrap,
          collapsable: false,
          children: [/*#__PURE__*/_jsx(Animated.View, {
            style: activeChildrenStyle,
            onLayout: onLayoutChildren,
            children: activeChildrenJSX
          }), /*#__PURE__*/_jsx(Animated.View, {
            style: inactiveChildrenStyle,
            onLayout: onLayoutChildren,
            children: inactiveChildrenJSX
          })]
        })]
      })
    })
  });
}
export default /*#__PURE__*/memo(Switch);
//# sourceMappingURL=switch.js.map