"use strict";

import { ArrowRightOutline } from '@fruits-chain/icons-react-native';
import React, { useState, useMemo, useRef } from 'react';
import { FlatList, View, Text } from 'react-native';
import Empty from "../empty/index.js";
import { getDefaultValue } from "../helpers/index.js";
import { useControllableValue, usePersistFn } from "../hooks/index.js";
import Search from "../search/index.js";
import Theme from "../theme/index.js";
import { varCreator, styleCreator } from "./style.js";
import TreeItem from "./tree-item.js";
import { TreeMultipleMode } from "./var.js";
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
export const findNodeByValue = (tree, value) => {
  for (const item of tree) {
    if (item.value === value) {
      return item;
    }
    if (item.children) {
      const _v = findNodeByValue(item.children, value);
      if (_v) {
        return _v;
      }
    }
  }
  return undefined;
};
export const findAllChildrenValue = tree => {
  const values = [];
  tree.forEach(item => {
    values.push(item.value);
    if (item.children?.length) {
      values.push(...findAllChildrenValue(item.children));
    }
  });
  return values;
};
export const findParentNodeByValue = (tree, value) => {
  for (const item of tree) {
    if (item.children?.length) {
      if (item.children.filter(i => i.value === value).length) {
        return item;
      } else {
        const _v = findParentNodeByValue(item.children, value);
        if (_v) {
          return _v;
        }
      }
    }
  }
  return null;
};
export const findAllParentNodeByValue = (tree, value) => {
  const nodes = [];
  const doFind = v => {
    const p = findParentNodeByValue(tree, v);
    if (p) {
      nodes.push(p);
      doFind(p.value);
    }
  };
  doFind(value);
  return nodes;
};
export const flattenDeepWidthChildren = tree => {
  const nodes = [];
  tree.forEach(item => {
    nodes.push(item);
    if (item.children?.length) {
      nodes.push(...flattenDeepWidthChildren(item.children));
    }
  });
  return nodes;
};
export const buildHighlightLabelConfig = (label, keyword) => {
  if (!keyword) {
    return [];
  }
  const _reg = new RegExp(keyword, 'gi');
  const results = [...label.matchAll(_reg)];
  if (results.length) {
    const nodes = [];
    let pointer = 0;
    results.forEach(res => {
      if (res.index !== pointer) {
        nodes.push({
          highlight: false,
          text: label.slice(pointer, res.index)
        });
      }
      pointer = (res.index || 0) + res[0].length;
      nodes.push({
        highlight: true,
        text: label.slice(res.index, pointer)
      });
    });
    if (pointer <= label.length - 1) {
      nodes.push({
        highlight: false,
        text: label.slice(pointer, label.length)
      });
    }
    return nodes;
  }
  return [];
};
const switcherIconWrapperStyle = {
  transform: [{
    rotateZ: '90deg'
  }]
};
const Tree = ({
  theme,
  multiple = false,
  multipleMode = TreeMultipleMode.NORMAL,
  options,
  renderSwitcherIcon,
  indent,
  activeColor,
  defaultExpandedValues,
  defaultExpandAll = false,
  search,
  onSearch,
  placeholder,
  minHeight = true,
  cancellable = false,
  editable = true,
  ...restProps
}) => {
  const [CV, STYLES] = Theme.useStyle({
    varCreator,
    styleCreator,
    theme
  });
  const [value, onChange] = useControllableValue(restProps, {
    defaultValue: multiple ? [] : undefined
  });
  const getOnSearch = usePersistFn(() => {
    return onSearch;
  });
  const [expandedValues, setExpandedValues] = useState(() => {
    if (defaultExpandAll) {
      return findAllChildrenValue(options);
    }
    const _values = [];
    /** 查看父节点是否要追加进去 */
    const doPushParent = subValue => {
      const _p = findParentNodeByValue(options, subValue);
      if (_p) {
        _values.push(_p.value);
        doPushParent(_p.value);
      }
    };
    defaultExpandedValues?.forEach(id => {
      _values.push(id);
      doPushParent(id);
    });
    return _values;
  });
  const listData = useMemo(() => {
    const nodes = [];
    const findVisibleItem = (treeOption, tier) => {
      const _nodes = [];
      if (expandedValues.indexOf(treeOption.value) > -1) {
        treeOption.children?.forEach(item => {
          _nodes.push({
            ...item,
            tier
          });
          _nodes.push(...findVisibleItem(item, tier + 1));
        });
      }
      return _nodes;
    };
    options.forEach(item => {
      nodes.push({
        ...item,
        tier: 0
      });
      nodes.push(...findVisibleItem(item, 1));
    });
    return nodes;
  }, [options, expandedValues]);
  const SearchFlatListRef = useRef(null);
  const [keyword, setKeyword] = useState('');
  const searchListData = useMemo(() => {
    const _onSearch = getOnSearch();
    const _options = flattenDeepWidthChildren(options);
    const _defaultFilter = () => {
      const nodes = [];
      _options.forEach(item => {
        const _labels = buildHighlightLabelConfig(item.label, keyword);
        if (_labels.length) {
          // 找到父级数据
          const parentNodes = findAllParentNodeByValue(options, item.value);
          if (parentNodes?.length) {
            _labels.push({
              highlight: false,
              text: ` / ${parentNodes.map(pn => pn.label).join(' / ')}`
            });
          }
          nodes.push({
            ...item,
            labels: _labels
          });
        }
      });
      return nodes;
    };
    return _onSearch ? _onSearch(keyword, _options) : _defaultFilter();
  }, [keyword, getOnSearch, options]);
  const _indent = getDefaultValue(indent, CV.tree_indent);
  const _activeColor = getDefaultValue(activeColor, CV.tree_active_color);
  const flatListStyle = useMemo(() => {
    if (minHeight === false) {
      return {};
    }
    if (typeof minHeight === 'number') {
      return {
        minHeight
      };
    }
    return {
      minHeight: CV.tree_min_height
    };
  }, [CV.tree_min_height, minHeight]);
  const onSearchKeyword = usePersistFn(t => {
    setKeyword(t);
    SearchFlatListRef.current?.scrollToIndex({
      index: 0
    });
  });
  const genOnPressItem = item => () => {
    if (multiple) {
      const valueTarget = value;
      const _value = valueTarget.filter(v => v !== item.value);
      const isInValue = _value.length !== valueTarget.length;
      const onChangeMultiple = values => {
        onChange(values, values.map(v => findNodeByValue(options, v)), {
          checked: !isInValue,
          option: item
        });
      };
      if (TreeMultipleMode.INDEPENDENT === multipleMode) {
        // 每个节点都是独立的
        if (isInValue) {
          onChangeMultiple(_value);
        } else {
          onChangeMultiple([..._value, item.value]);
        }
      } else {
        /** 所有子节点+自身节点 */
        const _nextValues = findAllChildrenValue(item.children || []);
        _nextValues.push(item.value);
        if (isInValue) {
          // 找到所有父节点
          // 追加当前及以后的节点
          const _ps = findAllParentNodeByValue(options, item.value).map(p => p.value).concat(_nextValues);

          // 向上查找父节点
          onChangeMultiple(_value.filter(v => _ps.indexOf(v) === -1));
        } else {
          // 先追加自身和子节点
          _nextValues.forEach(v => {
            if (_value.indexOf(v) === -1) {
              _value.push(v);
            }
          });

          /** 查看父节点是否要追加进去 */
          const doPushParent = subValue => {
            const _p = findParentNodeByValue(options, subValue);
            if (_p) {
              const _pNextValues = findAllChildrenValue(_p.children || []);
              // 父节点的所有子节点是否都在 _value
              if (_pNextValues.filter(pnv => _value.indexOf(pnv) === -1).length === 0) {
                _value.push(_p.value);
                doPushParent(_p.value);
              }
            }
          };
          doPushParent(item.value);
          onChangeMultiple(_value);
        }
      }
    } else {
      const _node = findNodeByValue(options, item.value);
      if (item.value !== value) {
        onChange(item.value, [_node], {
          checked: true,
          option: item
        });
      } else {
        if (cancellable) {
          onChange(null, [], {
            checked: false,
            option: item
          });
        }
      }
    }
  };
  return /*#__PURE__*/_jsxs(_Fragment, {
    children: [search ? /*#__PURE__*/_jsx(Search, {
      placeholder: placeholder,
      onSearch: onSearchKeyword,
      autoSearch: true
    }) : null, search && keyword ? searchListData.length ? /*#__PURE__*/_jsx(FlatList, {
      ref: SearchFlatListRef,
      keyboardShouldPersistTaps: "handled",
      bouncesZoom: false,
      contentContainerStyle: flatListStyle,
      data: searchListData,
      keyExtractor: item => `${item.value}`,
      renderItem: ({
        item
      }) => {
        const isActive = multiple ? value.indexOf(item.value) > -1 : value === item.value;
        return /*#__PURE__*/_jsx(TreeItem, {
          theme: theme,
          onPress: genOnPressItem(item),
          multiple: multiple,
          activeColor: _activeColor,
          indent: _indent,
          bold: item.bold,
          disabled: !editable || !!item.disabled,
          tier: 0,
          label: item.label,
          active: isActive,
          renderLabel: isActive ? undefined : p => {
            return /*#__PURE__*/_jsx(Text, {
              style: [STYLES.tree_item_text, p.disabled ? STYLES.tree_item_disabled_text : null, p.active || p.labelHighlight ? {
                color: p.activeColor
              } : null],
              numberOfLines: 1,
              children: item.labels.map((tObj, index) => {
                if (tObj.highlight) {
                  return /*#__PURE__*/_jsx(Text, {
                    style: STYLES.tree_item_highlight_text,
                    children: tObj.text
                  }, index);
                }
                return /*#__PURE__*/_jsx(React.Fragment, {
                  children: tObj.text
                }, index);
              })
            });
          }
        });
      }
    }, "search") : /*#__PURE__*/_jsx(Empty, {
      style: flatListStyle,
      full: true
    }) : /*#__PURE__*/_jsx(FlatList, {
      bouncesZoom: false,
      contentContainerStyle: flatListStyle,
      ListEmptyComponent: /*#__PURE__*/_jsx(Empty, {}),
      data: listData,
      keyExtractor: item => `${item.value}`,
      renderItem: ({
        item
      }) => {
        const isActive = multiple ? value.indexOf(item.value) > -1 : value === item.value;
        const _renderSwitcherIcon = item.renderSwitcherIcon || renderSwitcherIcon;
        const _onPressSwitcherIcon = () => {
          setExpandedValues(evs => {
            const _evs = evs.filter(v => v !== item.value);
            if (_evs.length !== evs.length) {
              return _evs;
            }
            return [..._evs, item.value];
          });
        };
        const _switcherIcon = item.children?.length ? _renderSwitcherIcon ? _renderSwitcherIcon({
          color: CV.tree_item_switcher_color,
          size: CV.tree_item_switcher_size
        }) : /*#__PURE__*/_jsx(ArrowRightOutline, {
          color: CV.tree_item_switcher_color,
          size: CV.tree_item_switcher_size
        }) : null;
        const _switcherIconJSX = expandedValues.indexOf(item.value) > -1 && item.switcherIconRotatable !== false ? /*#__PURE__*/_jsx(View, {
          style: switcherIconWrapperStyle,
          children: _switcherIcon
        }) : _switcherIcon;
        const _labelHighlight = findAllChildrenValue(item.children || []).filter(ic => {
          if (multiple) {
            return value.indexOf(ic) > -1;
          } else {
            return ic === value;
          }
        }).length > 0;
        return /*#__PURE__*/_jsx(TreeItem, {
          theme: theme,
          onPress: genOnPressItem(item),
          switcherIcon: _switcherIconJSX,
          switcherHighlight: item.switcherHighlight,
          multiple: multiple,
          activeColor: _activeColor,
          indent: _indent,
          bold: item.bold,
          disabled: !editable || !!item.disabled,
          tier: item.tier,
          label: item.label,
          active: isActive,
          renderLabel: item.render,
          labelHighlight: _labelHighlight,
          hasChildren: (item.children?.length || 0) > 0,
          onPressSwitcherIcon: _onPressSwitcherIcon
        });
      }
    }, "list")]
  });
};
export default Tree;
//# sourceMappingURL=tree.js.map